        PROGRAM MMUPDS2K
C!!! NASA Goddard Space Flight Center Open Source: Jan McGarry (Jan.McGarry@nasa.gov)
C ----------------------------------------------------------------
C Mound model update routines for STARCAL program: BATCH VERSION
C	Reads either:
C	  (1) the S2K MFT format (same as old MOBLAS format), or
C	  (2) the S2K operational format (same as New Controller format).
C ----------------------------------------------------------------
        IMPLICIT REAL*8 (A-H,O-Z)
        INCLUDE 'STARFILE.INC'
        TYPE (STAR_DATA_FILE_STRUCT) SDF
        PARAMETER (MNTTYP=0)
        PARAMETER (RAD = 57.295779513D0)
        INTEGER IYR,IMON,IDAY
        CHARACTER*30 FNAMA,FNAMX,FNAMP
        CHARACTER*10 CURDATE
        CHARACTER*8  UPD_TYPE
        CHARACTER    AOPT
        DIMENSION X0(NMMC),P0(NMMC,NMMC),PINIT(NMMC),MASK(NMMC)
C       ------------------------------------------------------------
        DATA FNAMA/'STARANAL.NEW'/,FNAMX/'COEFS.NEW'/,FNAMP/'COVAR.NEW'/
C       ------------------------------------------------------------
        IOPT=0
        DO I=1,200
           SDF%SD(I)%CATALOG_NUM=0
        END DO
        SDF%TEMP=13.5D0
        SDF%BARP=1003.7D0
        SDF%HUMD=50.D0
        SDF%CATALOG='FK5       '
        SDF%FILENAME(1:30)=' '
        CURDATE(1:10)=' '
	  CALL DATE_AND_TIME(CURDATE)
        SDF%FITTYPE=0
C       ------------------------------------------------------------
C
        PRINT *,'* STAR CALIBRATION ANALYSIS PROGRAM *'
        PRINT *,'ENTER THE STAR DATA FILE NAME: '
        READ(*,'(A)') SDF%FILENAME
	  PRINT *,'Enter (0) if this is MFT format,'
	  PRINT *,'   or (1) if this is operational format:'
	  READ(*,*) iformat
	  if(iformat.lt.0) iformat=0
	  if(iformat.gt.1) iformat=1
	  if(iformat.eq.0) then
	     print *,'MFT data format'
	  else
	     print *,'Operational data format'
	  endif
	  if(iformat.eq.1) then
           PRINT *,'INPUT COEF/COVAR SET-UP SELECTION:'
           PRINT *,' (0)==>set coeffs to zero, set covars to default'
           PRINT *,' (1)==>set coeffs & covars from star_data_file'
           PRINT *,'*(2)==>set coeffs from star_data_file and covars ',
     .                    'to default'
           PRINT *,' Enter selection [0/1/2]:'
           READ(*,*) IOPT
	  else
           PRINT *,'START STARCAL FROM SCRATCH (Y/N) ?'
           READ(*,'(A)') AOPT
           IF(AOPT.EQ.'Y'.OR.AOPT.EQ.'y') THEN
              IOPT=0
           ELSE
              PRINT *,'SCALE OR INITIALIZE COVARIANCE MATRIX [1/2] ?'
              READ(*,*) IOPT
              if(IOPT.lt.1) IOPT=1
	        if(IOPT.gt.2) IOPT=2
           ENDIF
	  endif
C
C               READ the star default file
        PRINT *,'Read the star default file'
        CALL DEF_READ(NMMC,SDF%SYSNAME,MAXSTR,UPD_TYPE,
     .                BSMAGL,BSMAGU,GSMAGL,GSMAGU,IASTAR,SDF%SIGR,
     .                SDF%ITER,SDF%SIGSCL,SDF%PSCALE,JOPT,PINIT,MASK)

        PRINT *,'Set initial coefficients and covariances'
C
C               SETUP for MFT format
	  if(iformat.eq.0) then
           IF(IOPT.EQ.1.OR.IOPT.EQ.2) THEN
              OPEN(UNIT=10,FILE='COEFS.OLD',STATUS='OLD')
              READ(10,*) IDUM
              DO I=1,NMMC
                 READ(10,*) SDF%X(I)
              ENDDO
              CLOSE(UNIT=10)
              OPEN(UNIT=10,FILE='COVAR.OLD',STATUS='OLD')
              READ(10,*) IDUM
              DO I=1,NMMC
                 DO J=I,NMMC
                    READ(10,*) SDF%P(I,J)
                 END DO
              END DO
              CLOSE(UNIT=10)
              DO I=1,NMMC
                 DO J=1,I
                    SDF%P(I,J)=SDF%P(J,I)
                 END DO
              END DO
           ENDIF
           PRINT *,'Set up of covariance matrix'
           CALL COVINI(IOPT,SDF,PINIT,MASK)
           DO I=1,NMMC
              DO J=1,NMMC
                 P0(I,J)=SDF%P(I,J)
              END DO
              WRITE(*,'(9(1X,F7.4))') (P0(I,J),J=1,NMMC)
              X0(I)=SDF%X(I)
           END DO
           OPEN(UNIT=10,FILE=SDF%FILENAME,STATUS='OLD')
	     READ(10,*,END=300) SDF%TEMP,SDF%BARP,SDF%HUMD
	  endif
C
C               SETUP for OPERATIONAL format
C               Read coefficients, covariances, etc from front
C                  of star_data_file.
	  if(iformat.eq.1) then
	     PRINT *,'Read the beginning of the star_data_file...'
           OPEN(UNIT=10,FILE=SDF%FILENAME,STATUS='OLD')
           READ(10,1111) ACHAR
           READ(10,1111) ACHAR
           READ(10,1111) ACHAR
1111       FORMAT(A1)
	     PRINT *,'Read the coefficients'
C               coefficients
           DO I=1,NMMC
                READ(10,*) SDF%X(I)
           END DO
           READ(10,1111) ACHAR
           READ(10,1111) ACHAR
	     PRINT *,'Read the covariances'
C               covariances
           DO I=1,NMMC
                READ(10,*) (SDF%P(I,J),J=I,NMMC)
           END DO
           DO I=1,NMMC
              DO J=1,I
                 SDF%P(I,J)=SDF%P(J,I)
              END DO
           END DO
           PRINT *,'Set up of covariance matrix'
           IF(IOPT.EQ.0) THEN
              DO I=1,NMMC
                 DO J=1,NMMC
                    SDF%P(I,J)=0.D0
                 END DO
                 SDF%X(I)=0.D0
                 SDF%P(I,I)=PINIT(I)
              END DO
           ELSEIF(IOPT.EQ.2) THEN
              DO I=1,NMMC
                 DO J=1,NMMC
                    SDF%P(I,J)=0.D0
                 END DO
                 SDF%P(I,I)=PINIT(I)
              END DO
           ENDIF
C               Save the starting coefficients and covariance matrix
           DO I=1,NMMC
              DO J=1,NMMC
                 P0(I,J)=SDF%P(I,J)
              END DO
              X0(I)=SDF%X(I)
           END DO
C
C               Pick up rest of information at the front of the
C               star_data_file:
C                       TEMP (in degrees C)
C                       BARP (in millibars)
C                       HUMD (in percent)
	     PRINT *,'Read the rest of the initial info from star data'
           READ(10,*) SDF%UT1_UTC
           READ(10,*) SDF%TEMP
           READ(10,*) SDF%BARP
           READ(10,*) SDF%HUMD
           READ(10,*) SIGR
           READ(10,*) SIGSCL
           READ(10,*) PSCALE
           PRINT *,' '
           PRINT *,'The following are values from the star_data_file:'
           PRINT *,'Meas. error (deg):    SIGR=',SIGR
           PRINT *,'STD scale factor:   SIGSCL=',SIGSCL
           PRINT *,'Covar.scale factor: PSCALE=',PSCALE
           PRINT *,' '
           PRINT *,'The following are values from the star_def file:'
           PRINT *,'Meas. error (deg):    SIGR=',SDF%SIGR
           PRINT *,'STD scale factor:   SIGSCL=',SDF%SIGSCL
           PRINT *,'Covar.scale factor: PSCALE=',SDF%PSCALE
           PRINT *,' '
333        PRINT *,'Do you want to use the values from:'
           PRINT *,'*(1) the star_data_file,'
           PRINT *,' (2) the star_default file,'
           PRINT *,' (3) enter your own values?'
           READ(*,*) KOPT
           IF(KOPT.LT.1.OR.KOPT.GT.3) GOTO 333
           IF(KOPT.EQ.1) THEN
              SDF%SIGR=SIGR
              SDF%SIGSCL=SIGSCL
              SDF%PSCALE=PSCALE
           ELSEIF(KOPT.EQ.3) THEN
              PRINT *,'Enter SIGR:'
              READ(*,*) SIGR
              PRINT *,'Enter SIGSCL:'
              READ(*,*) SIGSCL
              PRINT *,'Enter PSCALE:'
              READ(*,*) PSCALE
           ENDIF
	  endif
C
C               READ STAR DATA FILE
        PRINT *,'Getting star data'
        I=1
        OLDTIME=0.
 200    CONTINUE
	  if(iformat.eq.0) then
C               MFT format
           READ(10,*,END=300) LSTAR,IDOY,IHOUR,IMIN,ISEC,
     .                        SDF%SD(I)%A1,SDF%SD(I)%A2,
     .                        SDF%SD(I)%B1,SDF%SD(I)%B2,
     .                        SDF%SD(I)%D1,SDF%SD(I)%D2,NFK5,XMAG
 2020      FORMAT(1X,6(F11.8,1X))
           SDF%SD(I)%TOD%HRS=IHOUR
           SDF%SD(I)%TOD%MINS=IMIN
	     SDF%SD(I)%TOD%SECS=ISEC
           SDF%SD(I)%STARNUM=LSTAR
	     SDF%SD(I)%CATALOG_NUM=NFK5
	  else
           READ(10,2021,END=300) SDF%SD(I)%CATALOG_NUM,
     .                           SDF%SD(I)%STARNUM,
     .                           SDF%SD(I)%STAR_MAG,
     .                           SDF%SD(I)%SKY_SECTION,SDF%SD(I)%YEAR,
     .                           SDF%SD(I)%DOY,SDF%SD(I)%TOD%HRS,
     .                           SDF%SD(I)%TOD%MINS,
     .                           SDF%SD(I)%TOD%SECS,
     .                           SDF%SD(I)%A1,SDF%SD(I)%A2,
     .                           SDF%SD(I)%B1,SDF%SD(I)%B2,
     .                           SDF%SD(I)%D1,SDF%SD(I)%D2
 2021      FORMAT(1X,I5,1X,I3,1X,F4.0,1X,I2,31X,I4,7X,I3,1X,I2,1X,I2,1X,
     .            F5.2,6(1X,F8.4))
	  endif
	  SDF%SD(I)%STARNUM=SDF%SD(I)%STARNUM + 1
	  PRINT 2222,SDF%SD(I)%STARNUM,SDF%SD(I)%A1,SDF%SD(I)%A2,
     .             SDF%SD(I)%CATALOG_NUM,SDF%SD(I)%STAR_MAG
 2222   format(1x,i4,1x,f8.4,1x,f8.4,1x,i5,1x,f4.1)
        I=I+1
        GOTO 200
 300    CLOSE(UNIT=10)
        SDF%NSTARS=I-1
        PRINT *,'There are ',SDF%NSTARS,' stars.'
        IF(SDF%NSTARS.LT.2) THEN
           PRINT *,'* NOT ENOUGH STARS *'
           GOTO 900
        ENDIF
C
C               compute new mount model coefficients
        PRINT *,'Updating the mount model coefficients...'
        CALL BATCH(SDF,MNTTYP,IRETF)
        IF(IRETF.NE.0) THEN
           PRINT *,'* ERROR RETURN FROM BATCH *'
           GOTO 900
        ENDIF
C
        PRINT *,'Write out results to files...'
        CALL SUMMARY(SDF,X0,P0,MNTTYP,CURDATE)
        CALL COEWRT(NMMC,SDF%X,SDF%RMSTOT,FNAMX)
C               * VAX covariance write *
        OPEN(UNIT=10,FILE='COVAR.NEW',STATUS='UNKNOWN')
        WRITE(10,'(I3)') NMMC
        DO I=1,NMMC
           DO J=I,NMMC
              WRITE(10,*) SDF%P(I,J)
           END DO
        END DO
        CLOSE(UNIT=10)
        OPEN(UNIT=10,FILE='OLDNEW.SUM',STATUS='UNKNOWN')
        DO I=1,NMMC
           WRITE(10,*) X0(I),SDF%X(I)
        END DO
        CLOSE(UNIT=10)
        PRINT *,'The RMS(asec) of the calibration was ',
     .                  SDF%RMSTOT
        PRINT *,'The RMS(deg)  of the calibration was ',
     .                  SDF%RMSTOT/3600.
        PRINT *,'There were ',SDF%NOK,' stars used, and ',
     .                  SDF%NREJ,' rejected.'
C        
900     STOP
        END
